/**
 * @memberof module:apeman-wch/lib/watching
 * @function getWatches
 * @param {Apemanfile} apemanfile
 * @param {string[]} names
 * @param {function} callback
 */

"use strict";

var async = require('async'),
    findout = require('findout'),
    arrayreduce = require('arrayreduce');


/** @lends getWatches */
function getWatches(apemanfile, names, callback) {
    var arrayConcat = arrayreduce.arrayConcat();
    async.waterfall([
        function (callback) {
            names = [].concat(names || []).reduce(arrayConcat, []);
            async.concat(names, function (name, callback) {
                if (/#/.test(name)) {
                    getWatches.findChildWchs(apemanfile, name, callback);
                } else {
                    apemanfile.find("$wchs", name, callback);
                }
            }, callback);
        },
        function (watchs, callback) {
            watchs = watchs.reduce(arrayConcat, []);
            async.concat(watchs, function (watch, callback) {
                switch (typeof(watch)) {
                    case 'string':
                        var found = findout(watch, {safe: true});
                        if (found) {
                            callback(null, found);
                        } else {
                            getWatches(apemanfile, watch, callback);
                        }
                        break;
                    default:
                        callback(null, watch);
                        break;
                }
            }, callback);
        },
        function (watchs, callback) {
            watchs = watchs.reduce(arrayConcat, []);
            callback(null, watchs);
        }
    ], callback);
}

getWatches.findChildWchs = function (apemanfile, name, callback) {
    var namepaths = name.split(/#/g);
    var namespace = namepaths.shift();
    var $child = apemanfile.$children[namespace];
    if (!$child) {
        callback(new Error('$child not found with name: ' + name));
    }
    $child.find('$wchs', namepaths.shift(), function (err, found) {
        if (!err) {
            found.cwd = $child.$cwd;
        }
        callback(err, found);
    });
};

module.exports = getWatches;
