/**
 * List available watchs.
 * @memberof module:apeman-wch/lib/watching
 * @function listWatchs
 * @param {Apemanfile} - Apemanfile to work with.
 * @param {object} options - Optional settings.
 * @param {function} callback - Callback when done.
 */

"use strict";

var argx = require('argx'),
    async = require('async'),
    arrayreduce = require('arrayreduce'),
    ListLogger = require('../logging/list_logger');

/** @lends listWatchs */
function listWatchs(apemanfile, options, callback) {
    var args = argx(arguments);
    apemanfile = args.shift();
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    async.waterfall([
        function (callback) {
            apemanfile.find("$wchs", callback);
        },
        function (wchs, callback) {
            var $children = apemanfile.$children || {};
            async.concatSeries(Object.keys($children), function (name, callback) {
                var $child = $children[name];
                async.waterfall([
                    function (callback) {
                        $child.find("$wchs", function (err, found) {
                            // Ignore not found error.
                            callback(null, found || null);
                        });
                    },
                    function (found, callback) {
                        if (!found) {
                            callback(null, null);
                            return;
                        }
                        var result = {
                            namespace: name,
                            wchs: found,
                            cwd: $child.$cwd
                        };
                        callback(null, result);
                    }
                ], callback);
            }, function (err, childWchs) {
                if (!err) {
                    var logger = new ListLogger({
                        verbose: options.verbose
                    });
                    logger.logList(wchs, childWchs);
                }
                callback(err);
            });
        }
    ], callback);
}

module.exports = listWatchs;
