/**
 * Run watch.
 * @memberof module:apeman-wch/lib
 * @function runWatch
 */

"use strict";

var async = require('async'),
    arrayreduce = require('arrayreduce'),
    apeWatching = require('ape-watching'),
    getWatches = require('../watching/get_watches'),
    childProcess = require('child_process'),
    RunLogger = require('../logging/run_logger');

/** @lends runWatch */
function runWatch(apemanfile, name, options, callback) {
    var logger = new RunLogger({});
    logger.logWatchStarting(name);

    var interval = Number(options.interval || 300);
    async.waterfall([
        function (callback) {
            getWatches(apemanfile, name, callback);
        },
        function (watches, callback) {
            var timers = {};
            var watchers = [].concat(watches).map(function (watch) {
                var pattern = watch.pattern,
                    cwd = watch.cwd || process.cwd(),
                    actions = [].concat(watch.action).map(function (action) {
                        if (typeof(action) === 'string') {
                            action = function exec() {
                                childProcess.exec(action);
                            }
                        }
                        return action;
                    });
                return apeWatching.watchFiles(pattern, {
                    cwd: cwd,
                    prefix: 'apeman-wch'
                }, function (event, filename) {
                    var key = [event, filename].join('-');
                    clearTimeout(timers[key]);
                    timers[key] = setTimeout(function () {
                        actions.forEach(function (action) {
                            action(event, filename);
                        });
                    }, interval);
                });
            }).reduce(arrayreduce.arrayConcat(), []);
            watchers.closeAll = function () {
                watchers.forEach(function (watcher) {
                    watcher.close();
                });
            };
            callback(null, watchers);
        }
    ], function (err, watchers) {
        if (err) {
            logger.logWatchFailed(err);
        } else {
            logger.logWatchStarted();
        }
        callback(err, watchers);
    });

}

module.exports = runWatch;
